@@ -1,7 +1,3 @@
-class NoPullRequestError(Exception):
-    pass
-
-
 class RepositoryNotFoundError(Exception):
     pass
 
@@ -10,3 +6,7 @@ class ProcessGithubWebhookError(Exception):
     def __init__(self, err: dict[str, str]):
         self.err = err
         super().__init__(str(err))
+
+
+class NoApiTokenError(Exception):
+    pass
