@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import asyncio
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Coroutine
 
 from github.PullRequest import PullRequest
 
@@ -11,6 +11,7 @@ from webhook_server.libs.owners_files_handler import OwnersFileHandler
 from webhook_server.libs.runner_handler import RunnerHandler
 from webhook_server.utils.constants import (
     APPROVED_BY_LABEL_PREFIX,
+    AUTOMERGE_LABEL_STR,
     BRANCH_LABEL_PREFIX,
     BUILD_CONTAINER_STR,
     CAN_BE_MERGED_STR,
@@ -48,7 +49,9 @@ class PullRequestHandler:
         self.labels_handler = LabelsHandler(
             github_webhook=self.github_webhook, owners_file_handler=self.owners_file_handler
         )
-        self.check_run_handler = CheckRunHandler(github_webhook=self.github_webhook)
+        self.check_run_handler = CheckRunHandler(
+            github_webhook=self.github_webhook, owners_file_handler=self.owners_file_handler
+        )
         self.runner_handler = RunnerHandler(
             github_webhook=self.github_webhook, owners_file_handler=self.owners_file_handler
         )
@@ -56,14 +59,18 @@ class PullRequestHandler:
     async def process_pull_request_webhook_data(self, pull_request: PullRequest) -> None:
         hook_action: str = self.hook_data["action"]
         self.logger.info(f"{self.log_prefix} hook_action is: {hook_action}")
+        self.logger.debug(f"{self.log_prefix} pull_request: {pull_request.title} ({pull_request.number})")
 
         pull_request_data: dict[str, Any] = self.hook_data["pull_request"]
 
         if hook_action == "edited":
             await self.set_wip_label_based_on_title(pull_request=pull_request)
+            if self.hook_data["changes"].get("title"):
+                self.logger.info(f"{self.log_prefix} PR title changed, running conventional title check")
+                await self.runner_handler.run_conventional_title_check(pull_request=pull_request)
 
         if hook_action in ("opened", "reopened", "ready_for_review"):
-            tasks = []
+            tasks: list[Coroutine[Any, Any, Any]] = []
 
             if hook_action in ("opened", "ready_for_review"):
                 welcome_msg = self._prepare_welcome_comment()
@@ -76,18 +83,18 @@ class PullRequestHandler:
             results = await asyncio.gather(*tasks, return_exceptions=True)
             for result in results:
                 if isinstance(result, Exception):
-                    self.logger.error(f"{self.log_prefix} Async task failed: {results}")
+                    self.logger.error(f"{self.log_prefix} Async task failed: {result}")
 
             # Set auto merge only after all initialization of a new PR is done.
             await self.set_pull_request_automerge(pull_request=pull_request)
 
         if hook_action == "synchronize":
-            tasks = []
+            sync_tasks: list[Coroutine[Any, Any, Any]] = []
 
-            tasks.append(self.process_opened_or_synchronize_pull_request(pull_request=pull_request))
-            tasks.append(self.remove_labels_when_pull_request_sync(pull_request=pull_request))
+            sync_tasks.append(self.process_opened_or_synchronize_pull_request(pull_request=pull_request))
+            sync_tasks.append(self.remove_labels_when_pull_request_sync(pull_request=pull_request))
 
-            results = await asyncio.gather(*tasks, return_exceptions=True)
+            results = await asyncio.gather(*sync_tasks, return_exceptions=True)
 
             for result in results:
                 if isinstance(result, Exception):
@@ -126,13 +133,14 @@ class PullRequestHandler:
                 return
 
             self.logger.info(f"{self.log_prefix} PR {pull_request.number} {hook_action} with {labeled}")
+            self.logger.debug(f"PR labels are {pull_request.labels}")
 
             _split_label = labeled.split(LABELS_SEPARATOR, 1)
 
             if len(_split_label) == 2:
-                _lable_prefix, _user = _split_label
+                _label_prefix, _user = _split_label
 
-                if f"{_lable_prefix}{LABELS_SEPARATOR}" in (
+                if f"{_label_prefix}{LABELS_SEPARATOR}" in (
                     APPROVED_BY_LABEL_PREFIX,
                     LGTM_BY_LABEL_PREFIX,
                     CHANGED_REQUESTED_BY_LABEL_PREFIX,
@@ -144,17 +152,20 @@ class PullRequestHandler:
                         + self.owners_file_handler.root_approvers
                     ):
                         _check_for_merge = True
+                        self.logger.debug(f"PR approved by label action, will check for merge. user: {_user}")
 
             if self.github_webhook.verified_job and labeled_lower == VERIFIED_LABEL_STR:
                 _check_for_merge = True
+                self.logger.debug(f"PR verified label action, will check for merge. label: {labeled_lower}")
 
                 if action_labeled:
                     await self.check_run_handler.set_verify_check_success()
                 else:
                     await self.check_run_handler.set_verify_check_queued()
 
-            if labeled_lower in (WIP_STR, HOLD_LABEL_STR):
+            if labeled_lower in (WIP_STR, HOLD_LABEL_STR, AUTOMERGE_LABEL_STR):
                 _check_for_merge = True
+                self.logger.debug(f"PR has {labeled_lower} label, will check for merge.")
 
             if _check_for_merge:
                 await self.check_if_can_be_merged(pull_request=pull_request)
@@ -173,63 +184,115 @@ class PullRequestHandler:
     def _prepare_welcome_comment(self) -> str:
         self.logger.info(f"{self.log_prefix} Prepare welcome comment")
         supported_user_labels_str: str = "".join([f" * {label}\n" for label in USER_LABELS_DICT.keys()])
+
+        # Check if current user is auto-verified
+        is_auto_verified = self.github_webhook.parent_committer in self.github_webhook.auto_verified_and_merged_users
+        auto_verified_note = ""
+        if is_auto_verified:
+            auto_verified_note = """
+
+> **Note**: You are an auto-verified user. Your PRs will be automatically verified and may be auto-merged when all requirements are met.
+"""
+
+        # Check if issue creation is enabled
+        issue_creation_note = ""
+        if self.github_webhook.create_issue_for_new_pr:
+            issue_creation_note = "* **Issue Creation**: A tracking issue is created for this PR and will be closed when the PR is merged or closed\n"
+        else:
+            issue_creation_note = "* **Issue Creation**: Disabled for this repository\n"
+
         return f"""
 {self.github_webhook.issue_url_for_welcome_msg}
 
-The following are automatically added:
- * Add reviewers from OWNER file (in the root of the repository) under reviewers section.
- * Set PR size label.
- * New issue is created for the PR. (Closed when PR is merged/closed)
- * Run [pre-commit](https://pre-commit.ci/) if `.pre-commit-config.yaml` exists in the repo.
+## Welcome! 🎉
 
-Available user actions:
- * To mark PR as WIP comment `/wip` to the PR, To remove it from the PR comment `/wip cancel` to the PR.
- * To block merging of PR comment `/hold`, To un-block merging of PR comment `/hold cancel`.
- * To mark PR as verified comment `/verified` to the PR, to un-verify comment `/verified cancel` to the PR.
-        verified label removed on each new commit push.
- * To cherry pick a merged PR comment `/cherry-pick <target branch to cherry-pick to>` in the PR.
-    * Multiple target branches can be cherry-picked, separated by spaces. (`/cherry-pick branch1 branch2`)
-    * Cherry-pick will be started when PR is merged
- * To build and push container image command `/build-and-push-container` in the PR (tag will be the PR number).
-    * You can add extra args to the Podman build command
-        * Example: `/build-and-push-container --build-arg OPENSHIFT_PYTHON_WRAPPER_COMMIT=<commit_hash>`
- * To add a label by comment use `/<label name>`, to remove, use `/<label name> cancel`
- * To assign reviewers based on OWNERS file use `/assign-reviewers`
- * To check if PR can be merged use `/check-can-merge`
- * to assign reviewer to PR use `/assign-reviewer @<reviewer>`
+This pull request will be automatically processed with the following features:{auto_verified_note}
 
-PR will be approved when the following conditions are met:
- * `/approve` from one of the approvers.
- * Minimum number of required `/lgtm` (`{self.github_webhook.minimum_lgtm}`) is met.
+### 🔄 Automatic Actions
+* **Reviewer Assignment**: Reviewers are automatically assigned based on the OWNERS file in the repository root
+* **Size Labeling**: PR size labels (XS, S, M, L, XL, XXL) are automatically applied based on changes
+{issue_creation_note}* **Pre-commit Checks**: [pre-commit](https://pre-commit.ci/) runs automatically if `.pre-commit-config.yaml` exists
+* **Branch Labeling**: Branch-specific labels are applied to track the target branch
+* **Auto-verification**: Auto-verified users have their PRs automatically marked as verified
+
+### 📋 Available Commands
+
+#### PR Status Management
+* `/wip` - Mark PR as work in progress (adds WIP: prefix to title)
+* `/wip cancel` - Remove work in progress status
+* `/hold` - Block PR merging (approvers only)
+* `/hold cancel` - Unblock PR merging
+* `/verified` - Mark PR as verified
+* `/verified cancel` - Remove verification status
+
+#### Review & Approval
+* `/lgtm` - Approve changes (looks good to me)
+* `/approve` - Approve PR (approvers only)
+* `/automerge` - Enable automatic merging when all requirements are met (maintainers and approvers only)
+* `/assign-reviewers` - Assign reviewers based on OWNERS file
+* `/assign-reviewer @username` - Assign specific reviewer
+* `/check-can-merge` - Check if PR meets merge requirements
+
+#### Testing & Validation
+{self._prepare_retest_welcome_comment}
+
+#### Container Operations
+* `/build-and-push-container` - Build and push container image (tagged with PR number)
+  * Supports additional build arguments: `/build-and-push-container --build-arg KEY=value`
+
+#### Cherry-pick Operations
+* `/cherry-pick <branch>` - Schedule cherry-pick to target branch when PR is merged
+  * Multiple branches: `/cherry-pick branch1 branch2 branch3`
+
+#### Label Management
+* `/<label-name>` - Add a label to the PR
+* `/<label-name> cancel` - Remove a label from the PR
+
+### ✅ Merge Requirements
+
+This PR will be automatically approved when the following conditions are met:
+
+1. **Approval**: `/approve` from at least one approver
+2. **LGTM Count**: Minimum {self.github_webhook.minimum_lgtm} `/lgtm` from reviewers
+3. **Status Checks**: All required status checks must pass
+4. **No Blockers**: No WIP, hold, or conflict labels
+5. **Verified**: PR must be marked as verified (if verification is enabled)
+
+### 📊 Review Process
 
 <details>
-<summary>Approvers and Reviewers</summary>
+<summary><strong>Approvers and Reviewers</strong></summary>
 
 {self._prepare_owners_welcome_comment()}
 </details>
 
 <details>
-<summary>Supported /retest check runs</summary>
-
-{self._prepare_retest_welcome_comment}
-</details>
-
-<details>
-<summary>Supported labels</summary>
+<summary><strong>Available Labels</strong></summary>
 
 {supported_user_labels_str}
 </details>
+
+### 💡 Tips
+
+* **WIP Status**: Use `/wip` when your PR is not ready for review
+* **Verification**: The verified label is automatically removed on each new commit
+* **Cherry-picking**: Cherry-pick labels are processed when the PR is merged
+* **Container Builds**: Container images are automatically tagged with the PR number
+* **Permission Levels**: Some commands require approver permissions
+* **Auto-verified Users**: Certain users have automatic verification and merge privileges
+
+For more information, please refer to the project documentation or contact the maintainers.
     """
 
     def _prepare_owners_welcome_comment(self) -> str:
-        body_approvers: str = " * Approvers:\n"
-        body_reviewers: str = " * Reviewers:\n"
+        body_approvers: str = "**Approvers:**\n"
+        body_reviewers: str = "**Reviewers:**\n"
 
         for _approver in self.owners_file_handler.all_pull_request_approvers:
-            body_approvers += f"   * {_approver}\n"
+            body_approvers += f" * {_approver}\n"
 
         for _reviewer in self.owners_file_handler.all_pull_request_reviewers:
-            body_reviewers += f"   * {_reviewer}\n"
+            body_reviewers += f" * {_reviewer}\n"
 
         return f"""
 {body_approvers}
@@ -240,25 +303,26 @@ PR will be approved when the following conditions are met:
     @property
     def _prepare_retest_welcome_comment(self) -> str:
         retest_msg: str = ""
+
         if self.github_webhook.tox:
-            retest_msg += f" * `/retest {TOX_STR}`: Retest tox\n"
+            retest_msg += f" * `/retest {TOX_STR}` - Run Python test suite with tox\n"
 
         if self.github_webhook.build_and_push_container:
-            retest_msg += f" * `/retest {BUILD_CONTAINER_STR}`: Retest build-container\n"
+            retest_msg += f" * `/retest {BUILD_CONTAINER_STR}` - Rebuild and test container image\n"
 
         if self.github_webhook.pypi:
-            retest_msg += f" * `/retest {PYTHON_MODULE_INSTALL_STR}`: Retest python-module-install\n"
+            retest_msg += f" * `/retest {PYTHON_MODULE_INSTALL_STR}` - Test Python package installation\n"
 
         if self.github_webhook.pre_commit:
-            retest_msg += f" * `/retest {PRE_COMMIT_STR}`: Retest pre-commit\n"
+            retest_msg += f" * `/retest {PRE_COMMIT_STR}` - Run pre-commit hooks and checks\n"
 
         if self.github_webhook.conventional_title:
-            retest_msg += f" * `/retest {CONVENTIONAL_TITLE_STR}`: Retest conventional-title\n"
+            retest_msg += f" * `/retest {CONVENTIONAL_TITLE_STR}` - Validate commit message format\n"
 
         if retest_msg:
-            retest_msg += " * `/retest all`: Retest all\n"
+            retest_msg += " * `/retest all` - Run all available tests\n"
 
-        return " * This repository does not support retest actions" if not retest_msg else retest_msg
+        return " * No retest actions are configured for this repository" if not retest_msg else retest_msg
 
     async def label_all_opened_pull_requests_merge_state_after_merged(self) -> None:
         """
@@ -276,6 +340,7 @@ PR will be approved when the following conditions are met:
             await self.label_pull_request_by_merge_state(pull_request=pull_request)
 
     async def delete_remote_tag_for_merged_or_closed_pr(self, pull_request: PullRequest) -> None:
+        self.logger.debug(f"{self.log_prefix} Checking if need to delete remote tag for {pull_request.number}")
         if not self.github_webhook.build_and_push_container:
             self.logger.info(f"{self.log_prefix} repository do not have container configured")
             return
@@ -349,15 +414,13 @@ PR will be approved when the following conditions are met:
                 break
 
     async def process_opened_or_synchronize_pull_request(self, pull_request: PullRequest) -> None:
-        tasks = []
+        tasks: list[Coroutine[Any, Any, Any]] = []
 
         tasks.append(self.owners_file_handler.assign_reviewers(pull_request=pull_request))
         tasks.append(
             self.labels_handler._add_label(
-                **{
-                    "pull_request": pull_request,
-                    "label": f"{BRANCH_LABEL_PREFIX}{pull_request.base.ref}",
-                },
+                pull_request=pull_request,
+                label=f"{BRANCH_LABEL_PREFIX}{pull_request.base.ref}",
             )
         )
         tasks.append(self.label_pull_request_by_merge_state(pull_request=pull_request))
@@ -386,6 +449,10 @@ PR will be approved when the following conditions are met:
                 self.logger.error(f"{self.log_prefix} Async task failed: {result}")
 
     async def create_issue_for_new_pull_request(self, pull_request: PullRequest) -> None:
+        if not self.github_webhook.create_issue_for_new_pr:
+            self.logger.info(f"{self.log_prefix} Issue creation for new PRs is disabled for this repository")
+            return
+
         if self.github_webhook.parent_committer in self.github_webhook.auto_verified_and_merged_users:
             self.logger.info(
                 f"{self.log_prefix} Committer {self.github_webhook.parent_committer} is part of "
@@ -408,10 +475,17 @@ PR will be approved when the following conditions are met:
         return f"[Auto generated]\nNumber: [#{pull_request.number}]"
 
     async def set_pull_request_automerge(self, pull_request: PullRequest) -> None:
-        auto_merge = (
-            pull_request.base.ref in self.github_webhook.set_auto_merge_prs
-            or self.github_webhook.parent_committer in self.github_webhook.auto_verified_and_merged_users
+        set_auto_merge_base_branch = pull_request.base.ref in self.github_webhook.set_auto_merge_prs
+        self.logger.debug(f"{self.log_prefix} set auto merge for base branch is {set_auto_merge_base_branch}")
+        parent_committer_in_auto_merge_users = (
+            self.github_webhook.parent_committer in self.github_webhook.auto_verified_and_merged_users
         )
+        self.logger.debug(
+            f"{self.log_prefix} parent committer {self.github_webhook.parent_committer} in auto merge users is "
+            f"{parent_committer_in_auto_merge_users}"
+        )
+
+        auto_merge = set_auto_merge_base_branch or parent_committer_in_auto_merge_users
 
         self.logger.debug(f"{self.log_prefix} auto_merge: {auto_merge}, branch: {pull_request.base.ref}")
 
@@ -431,7 +505,7 @@ PR will be approved when the following conditions are met:
                 self.logger.error(f"{self.log_prefix} Exception while setting auto merge: {exp}")
 
     async def remove_labels_when_pull_request_sync(self, pull_request: PullRequest) -> None:
-        tasks = []
+        tasks: list[Coroutine[Any, Any, Any]] = []
         for _label in pull_request.labels:
             _label_name = _label.name
             if (
@@ -442,10 +516,8 @@ PR will be approved when the following conditions are met:
             ):
                 tasks.append(
                     self.labels_handler._remove_label(
-                        **{
-                            "pull_request": pull_request,
-                            "label": _label_name,
-                        },
+                        pull_request=pull_request,
+                        label=_label_name,
                     )
                 )
 
@@ -531,6 +603,7 @@ PR will be approved when the following conditions are met:
             self.logger.debug(f"{self.log_prefix} check if can be merged. PR labels are: {_labels}")
 
             is_pr_mergable = pull_request.mergeable
+            self.logger.debug(f"{self.log_prefix} PR mergeable is {is_pr_mergable}")
             if not is_pr_mergable:
                 failure_output += f"PR is not mergeable: {is_pr_mergable}\n"
 
@@ -542,10 +615,12 @@ PR will be approved when the following conditions are met:
             )
             if required_check_in_progress_failure_output:
                 failure_output += required_check_in_progress_failure_output
+            self.logger.debug(f"{self.log_prefix} required_check_in_progress_failure_output: {failure_output}")
 
             labels_failure_output = self.labels_handler.wip_or_hold_lables_exists(labels=_labels)
             if labels_failure_output:
                 failure_output += labels_failure_output
+            self.logger.debug(f"{self.log_prefix} wip_or_hold_lables_exists: {failure_output}")
 
             required_check_failed_failure_output = await self.check_run_handler.required_check_failed_or_no_status(
                 pull_request=pull_request,
@@ -554,14 +629,17 @@ PR will be approved when the following conditions are met:
             )
             if required_check_failed_failure_output:
                 failure_output += required_check_failed_failure_output
+            self.logger.debug(f"{self.log_prefix} required_check_failed_or_no_status: {failure_output}")
 
-            labels_failure_output = self._check_lables_for_can_be_merged(labels=_labels)
+            labels_failure_output = self._check_labels_for_can_be_merged(labels=_labels)
             if labels_failure_output:
                 failure_output += labels_failure_output
+            self.logger.debug(f"{self.log_prefix} _check_labels_for_can_be_merged: {failure_output}")
 
             pr_approvered_failure_output = await self._check_if_pr_approved(labels=_labels)
             if pr_approvered_failure_output:
                 failure_output += pr_approvered_failure_output
+            self.logger.debug(f"{self.log_prefix} _check_if_pr_approved: {failure_output}")
 
             if not failure_output:
                 await self.labels_handler._add_label(pull_request=pull_request, label=CAN_BE_MERGED_STR)
@@ -586,6 +664,7 @@ PR will be approved when the following conditions are met:
 
     async def _check_if_pr_approved(self, labels: list[str]) -> str:
         self.logger.info(f"{self.log_prefix} Check if pull request is approved by pull request labels.")
+        self.logger.debug(f"labels are {labels}")
 
         error: str = ""
         approved_by = []
@@ -596,9 +675,11 @@ PR will be approved when the following conditions are met:
             + self.owners_file_handler.root_approvers.copy()
             + self.owners_file_handler.root_reviewers.copy()
         )
+        self.logger.debug(f"all_reviewers: {all_reviewers}")
         all_reviewers_without_pr_owner = {
             _reviewer for _reviewer in all_reviewers if _reviewer != self.github_webhook.parent_committer
         }
+        self.logger.debug(f"all_reviewers_without_pr_owner: {all_reviewers_without_pr_owner}")
 
         all_reviewers_without_pr_owner_and_lgtmed = all_reviewers_without_pr_owner.copy()
 
@@ -608,13 +689,17 @@ PR will be approved when the following conditions are met:
                 if LGTM_BY_LABEL_PREFIX.lower() in _label.lower() and reviewer in all_reviewers_without_pr_owner:
                     lgtm_count += 1
                     all_reviewers_without_pr_owner_and_lgtmed.remove(reviewer)
+        self.logger.debug(f"lgtm_count: {lgtm_count}")
 
         for _label in labels:
             if APPROVED_BY_LABEL_PREFIX.lower() in _label.lower():
                 approved_by.append(_label.split(LABELS_SEPARATOR)[-1])
+        self.logger.debug(f"approved_by: {approved_by}")
 
         missing_approvers = list(set(self.owners_file_handler.all_pull_request_approvers.copy()))
+        self.logger.debug(f"missing_approvers: {missing_approvers}")
         owners_data_changed_files = await self.owners_file_handler.owners_data_for_changed_files()
+        self.logger.debug(f"owners_data_changed_files: {owners_data_changed_files}")
 
         # If any of root approvers is in approved_by list, the pull request is approved
         for _approver in approved_by:
@@ -636,6 +721,7 @@ PR will be approved when the following conditions are met:
                         break
 
         missing_approvers = list(set(missing_approvers))
+        self.logger.debug(f"missing_approvers after check: {missing_approvers}")
 
         if missing_approvers:
             error += f"Missing approved from approvers: {', '.join(missing_approvers)}\n"
@@ -653,8 +739,8 @@ PR will be approved when the following conditions are met:
 
         return error
 
-    def _check_lables_for_can_be_merged(self, labels: list[str]) -> str:
-        self.logger.debug(f"{self.log_prefix} _check_lables_for_can_be_merged.")
+    def _check_labels_for_can_be_merged(self, labels: list[str]) -> str:
+        self.logger.debug(f"{self.log_prefix} _check_labels_for_can_be_merged.")
         failure_output = ""
 
         for _label in labels:
@@ -662,11 +748,13 @@ PR will be approved when the following conditions are met:
                 change_request_user = _label.split(LABELS_SEPARATOR)[-1]
                 if change_request_user in self.owners_file_handler.all_pull_request_approvers:
                     failure_output += "PR has changed requests from approvers\n"
+                    self.logger.debug(f"Found changed request by {change_request_user}")
 
         missing_required_labels = []
         for _req_label in self.github_webhook.can_be_merged_required_labels:
             if _req_label not in labels:
                 missing_required_labels.append(_req_label)
+                self.logger.debug(f"Missing required label {_req_label}")
 
         if missing_required_labels:
             failure_output += f"Missing required labels: {', '.join(missing_required_labels)}\n"
