@@ -1,9 +1,20 @@
+import pytest
 import yaml
 
-import pytest
+from webhook_server.libs.pull_request_handler import PullRequestHandler
 from webhook_server.tests.conftest import ContentFile, Tree
 from webhook_server.utils.constants import APPROVED_BY_LABEL_PREFIX
 
+ALL_CHANGED_FILES = [
+    "OWNERS",
+    "folder1/OWNERS",
+    "code/file.py",
+    "README.md",
+    "folder2/lib.py",
+    "folder/folder4/another_file.txt",
+    "folder5/file",
+]
+
 
 class Repository:
     def __init__(self):
@@ -12,7 +23,7 @@ class Repository:
     def get_git_tree(self, sha: str, recursive: bool):
         return Tree("")
 
-    def get_contents(self, path: str, ref):
+    def get_contents(self, path: str, ref: str):
         owners_data = yaml.dump({
             "approvers": ["root_approver1", "root_approver2"],
             "reviewers": ["root_reviewer1", "root_reviewer2"],
@@ -53,8 +64,17 @@ class Repository:
 
 
 @pytest.fixture(scope="function")
-def all_approvers_and_reviewers(process_github_webhook):
-    process_github_webhook.all_approvers_and_reviewers = {
+def changed_files(request, owners_file_handler):
+    if hasattr(request, "param") and request.param:
+        owners_file_handler.changed_files = request.param[0]
+
+    else:
+        owners_file_handler.changed_files = ALL_CHANGED_FILES
+
+
+@pytest.fixture(scope="function")
+def all_repository_approvers_and_reviewers(owners_file_handler):
+    owners_file_handler.all_repository_approvers_and_reviewers = {
         ".": {"approvers": ["root_approver1", "root_approver2"], "reviewers": ["root_reviewer1", "root_reviewer2"]},
         "folder1": {
             "approvers": ["folder1_approver1", "folder1_approver2"],
@@ -74,8 +94,8 @@ def all_approvers_and_reviewers(process_github_webhook):
 
 
 @pytest.fixture(scope="function")
-def all_approvers_reviewers(process_github_webhook):
-    process_github_webhook.all_approvers = [
+def all_approvers_reviewers(owners_file_handler):
+    owners_file_handler.all_pull_request_approvers = [
         "folder1_approver1",
         "folder1_approver2",
         "folder4_approver1",
@@ -86,9 +106,9 @@ def all_approvers_reviewers(process_github_webhook):
         "root_approver2",
     ]
 
-    process_github_webhook.all_approvers.sort()
+    owners_file_handler.all_pull_request_approvers.sort()
 
-    process_github_webhook.all_reviewers = [
+    owners_file_handler.all_pull_request_reviewers = [
         "folder1_reviewer1",
         "folder1_reviewer2",
         "folder4_reviewer1",
@@ -99,17 +119,23 @@ def all_approvers_reviewers(process_github_webhook):
         "root_reviewer2",
     ]
 
-    process_github_webhook.all_reviewers.sort()
+    owners_file_handler.all_pull_request_reviewers.sort()
 
 
-def test_get_all_approvers_and_reviewers(process_github_webhook, all_approvers_and_reviewers):
+@pytest.mark.asyncio
+async def test_get_all_repository_approvers_and_reviewers(
+    changed_files, process_github_webhook, owners_file_handler, pull_request, all_repository_approvers_and_reviewers
+):
     process_github_webhook.repository = Repository()
-    read_owners_result = process_github_webhook.get_all_approvers_and_reviewers()
-    assert read_owners_result == process_github_webhook.all_approvers_and_reviewers
+    read_owners_result = await owners_file_handler.get_all_repository_approvers_and_reviewers(pull_request=pull_request)
+    assert read_owners_result == owners_file_handler.all_repository_approvers_and_reviewers
 
 
-def test_owners_data_for_changed_files(process_github_webhook, all_approvers_and_reviewers):
-    owners_data_chaged_files_result = process_github_webhook.owners_data_for_changed_files()
+@pytest.mark.asyncio
+async def test_owners_data_for_changed_files(
+    changed_files, process_github_webhook, owners_file_handler, all_repository_approvers_and_reviewers
+):
+    owners_data_changed_files_result = await owners_file_handler.owners_data_for_changed_files()
     owners_data_chaged_files_expected = {
         "folder5": {
             "approvers": ["folder5_approver1", "folder5_approver2"],
@@ -128,21 +154,39 @@ def test_owners_data_for_changed_files(process_github_webhook, all_approvers_and
         },
     }
 
-    assert owners_data_chaged_files_result == owners_data_chaged_files_expected
+    assert owners_data_changed_files_result == owners_data_chaged_files_expected
 
 
-def test_all_approvers_reviewers(process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers):
-    all_approvers = process_github_webhook.get_all_approvers()
-    assert all_approvers == process_github_webhook.all_approvers
+@pytest.mark.asyncio
+async def test_all_approvers_reviewers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
+):
+    all_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    assert all_approvers == owners_file_handler.all_pull_request_approvers
 
-    all_reviewers = process_github_webhook.get_all_reviewers()
-    assert all_reviewers == process_github_webhook.all_reviewers
+    all_pull_request_reviewers = await owners_file_handler.get_all_pull_request_reviewers()
+    assert all_pull_request_reviewers == owners_file_handler.all_pull_request_reviewers
 
 
-def test_check_pr_approved(process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+@pytest.mark.asyncio
+async def test_check_pr_approved(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
+):
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook,
+        owners_file_handler=owners_file_handler,
+    )
+    process_github_webhook.parent_committer = "test"
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
             f"{APPROVED_BY_LABEL_PREFIX}root_approver2",
@@ -157,9 +201,20 @@ def test_check_pr_approved(process_github_webhook, all_approvers_and_reviewers,
     assert check_if_pr_approved == ""
 
 
-def test_check_pr_minimum_approved(process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+@pytest.mark.asyncio
+async def test_check_pr_minimum_approved(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
+):
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook,
+        owners_file_handler=owners_file_handler,
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
             f"{APPROVED_BY_LABEL_PREFIX}folder1_approver1",
@@ -170,18 +225,29 @@ def test_check_pr_minimum_approved(process_github_webhook, all_approvers_and_rev
     assert check_if_pr_approved == ""
 
 
-def test_check_pr_not_approved(process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+@pytest.mark.asyncio
+async def test_check_pr_not_approved(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
+):
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook,
+        owners_file_handler=owners_file_handler,
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
-            f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
+            f"{APPROVED_BY_LABEL_PREFIX}folder1_approver1",
         ]
     )
     missing_approvers = [appr.strip() for appr in check_if_pr_approved.split(":")[-1].strip().split(",")]
     missing_approvers.sort()
     expected_approvers = [
-        "folder1_approver1",
-        "folder1_approver2",
+        "root_approver1",
+        "root_approver2",
         "folder4_approver1",
         "folder4_approver2",
         "folder5_approver1",
@@ -191,21 +257,29 @@ def test_check_pr_not_approved(process_github_webhook, all_approvers_and_reviewe
     assert missing_approvers == expected_approvers
 
 
-def test_check_pr_partial_approved(process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+@pytest.mark.asyncio
+async def test_check_pr_partial_approved(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
+):
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
-            f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
-            f"{APPROVED_BY_LABEL_PREFIX}root_approver2",
+            f"{APPROVED_BY_LABEL_PREFIX}folder1_approver1",
+            f"{APPROVED_BY_LABEL_PREFIX}folder4_approver2",
         ]
     )
     missing_approvers = [appr.strip() for appr in check_if_pr_approved.split(":")[-1].strip().split(",")]
     missing_approvers.sort()
     expected_approvers = [
-        "folder1_approver1",
-        "folder1_approver2",
-        "folder4_approver1",
-        "folder4_approver2",
+        "root_approver1",
+        "root_approver2",
         "folder5_approver1",
         "folder5_approver2",
     ]
@@ -214,7 +288,7 @@ def test_check_pr_partial_approved(process_github_webhook, all_approvers_and_rev
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -226,11 +300,19 @@ def test_check_pr_partial_approved(process_github_webhook, all_approvers_and_rev
     ],
     indirect=True,
 )
-def test_check_pr_approved_specific_folder(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_specific_folder(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
             f"{APPROVED_BY_LABEL_PREFIX}folder4_approver1",
@@ -240,7 +322,7 @@ def test_check_pr_approved_specific_folder(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -252,11 +334,19 @@ def test_check_pr_approved_specific_folder(
     ],
     indirect=True,
 )
-def test_check_pr_approved_nested_folder_no_owners(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_nested_folder_no_owners(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
         ]
@@ -265,7 +355,7 @@ def test_check_pr_approved_nested_folder_no_owners(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -275,11 +365,19 @@ def test_check_pr_approved_nested_folder_no_owners(
     ],
     indirect=True,
 )
-def test_check_pr_approved_specific_folder_with_root_approvers(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_specific_folder_with_root_approvers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}folder1_approver1",
         ]
@@ -295,7 +393,7 @@ def test_check_pr_approved_specific_folder_with_root_approvers(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -305,11 +403,19 @@ def test_check_pr_approved_specific_folder_with_root_approvers(
     ],
     indirect=True,
 )
-def test_check_pr_approved_specific_folder_no_root_approvers(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_specific_folder_no_root_approvers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}folder5_approver1",
         ]
@@ -318,7 +424,7 @@ def test_check_pr_approved_specific_folder_no_root_approvers(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -328,11 +434,19 @@ def test_check_pr_approved_specific_folder_no_root_approvers(
     ],
     indirect=True,
 )
-def test_check_pr_not_approved_specific_folder_without_owners(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_not_approved_specific_folder_without_owners(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}folder5_approver1",
         ]
@@ -348,7 +462,7 @@ def test_check_pr_not_approved_specific_folder_without_owners(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -358,11 +472,19 @@ def test_check_pr_not_approved_specific_folder_without_owners(
     ],
     indirect=True,
 )
-def test_check_pr_approved_specific_folder_without_owners(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_specific_folder_without_owners(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
         ]
@@ -371,7 +493,7 @@ def test_check_pr_approved_specific_folder_without_owners(
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -382,11 +504,19 @@ def test_check_pr_approved_specific_folder_without_owners(
     ],
     indirect=True,
 )
-def test_check_pr_approved_folder_with_no_owners_and_folder_without_root_approvers(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_folder_with_no_owners_and_folder_without_root_approvers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
             f"{APPROVED_BY_LABEL_PREFIX}folder5_approver1",
@@ -396,7 +526,7 @@ def test_check_pr_approved_folder_with_no_owners_and_folder_without_root_approve
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -407,11 +537,19 @@ def test_check_pr_approved_folder_with_no_owners_and_folder_without_root_approve
     ],
     indirect=True,
 )
-def test_check_pr_not_approved_folder_with_no_owners_and_folder_without_root_approvers(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_not_approved_folder_with_no_owners_and_folder_without_root_approvers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}folder5_approver1",
         ]
@@ -427,37 +565,7 @@ def test_check_pr_not_approved_folder_with_no_owners_and_folder_without_root_app
 
 
 @pytest.mark.parametrize(
-    "process_github_webhook",
-    [
-        pytest.param([
-            [
-                "folder1/sub_folder1/file",
-            ]
-        ])
-    ],
-    indirect=True,
-)
-def test_check_pr_not_approved_subfolder_with_owners(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
-):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
-        labels=[
-            f"{APPROVED_BY_LABEL_PREFIX}root_approver1",
-        ]
-    )
-    missing_approvers = [appr.strip() for appr in check_if_pr_approved.split(":")[-1].strip().split(",")]
-    missing_approvers.sort()
-    expected_approvers = [
-        "folder1_approver1",
-        "folder1_approver2",
-    ]
-    expected_approvers.sort()
-    assert missing_approvers == expected_approvers
-
-
-@pytest.mark.parametrize(
-    "process_github_webhook",
+    "changed_files",
     [
         pytest.param([
             [
@@ -467,11 +575,19 @@ def test_check_pr_not_approved_subfolder_with_owners(
     ],
     indirect=True,
 )
-def test_check_pr_approved_subfolder_with_owners_no_root_approvers(
-    process_github_webhook, all_approvers_and_reviewers, all_approvers_reviewers
+@pytest.mark.asyncio
+async def test_check_pr_approved_subfolder_with_owners_no_root_approvers(
+    changed_files,
+    process_github_webhook,
+    owners_file_handler,
+    all_repository_approvers_and_reviewers,
+    all_approvers_reviewers,
 ):
-    process_github_webhook.all_approvers = process_github_webhook.get_all_approvers()
-    check_if_pr_approved = process_github_webhook._check_if_pr_approved(
+    owners_file_handler.all_pull_request_approvers = await owners_file_handler.get_all_pull_request_approvers()
+    pull_request_handler = PullRequestHandler(
+        github_webhook=process_github_webhook, owners_file_handler=owners_file_handler
+    )
+    check_if_pr_approved = await pull_request_handler._check_if_pr_approved(
         labels=[
             f"{APPROVED_BY_LABEL_PREFIX}folder5_approver1",
         ]
